/****************************************************************************/
/***** Copyright (c) Brooktree Corporation, 1996. All Rights Reserved *******/
/****************************************************************************/
/*                                                                          */
/*                      BROOKTREE CONFIDENTIAL                              */
/*                                                                          */
/* Filename:    CCDEC.H                                                     */
/*                                                                          */
/* Description: Closed caption decode in software include file.             */
/*                                                                          */
/*              Main internal include file, holds object typedef.           */
/*                                                                          */
/* Author:      Dave Tharp, based loosely on previous work by               */
/*                Steve Mair, and Steve Glennon.                            */
/*                                                                          */
/****************************************************************************/
/* PVCS Muck:
 * $Header: G:/SwDev/W95/Video/pisces/dvlpmnt/CC/rcs/CCDEC.H 2.6 1997/06/27 17:03:39 dtharp Released $
 * $Revision: 2.6 $
 * $Date: 1997/06/27 17:03:39 $
 * $Author: dtharp $
 * $Archive:   R:\rcapsvs\ccdec.h_v  $
 * $Log: CCDEC.H $
 * Revision 2.6  1997/06/27 17:03:39  dtharp
 * Moved global variables and functions into class in attempt to solve
 * re-entrancy problem.
 * Revision 2.5  1997/04/18 00:35:39  dtharp
 * Fix for Bug Id 433. Closed Caption Performance (speed, cpu).
 * Revision 2.4.1.1  1997/04/15 23:53:28  dtharp
 * Bug Id 434: CC decoder performance tune.
 * Revision 2.4  1997/04/03 16:40:46  dtharp
 * Fix for Instrumented build only. Does not affect "real" builds. 
 * DumpWaveform() prototype was incorrect.
 * Revision 2.3  1997/01/17 01:18:15  dtharp
 * Merged new cc decoder branch.
 * Revision 2.2.1.5  1996/12/10 23:15:45  dtharp
 * Changed m_bitWidth from int to DWORD.
 * Revision 2.2.1.4  1996/12/07 00:57:30  dtharp
 * Cleaned up all unneeded variables.
 * Revision 2.2.1.3  1996/12/06 20:14:38  dtharp
 * Cleaned out old decoder helper functions.
 * Revision 2.2.1.2  1996/12/06 00:09:02  trantq
 * Added methods OutputInstrumentation() and DumpWaveForm().
 * Revision 2.2.1.1  1996/12/02 22:54:53  trantq
 * Implement CC instrumentation.
 * Revision 2.2  1996/10/10 20:20:38  dtharp
 * Removed BT819CC constants to cc_api.h
 * Revision 2.1  1996/06/10 20:12:08  ssm
 * MarkBeta1AsRev2
 * 
 *    Rev 1.2   06 Mar 1996 11:31:50   glennon
 * Added prototype for CheckRuninPresent.
 *
 *    Rev 1.1   05 Mar 1996 17:44:44   glennon
 * Added member function for ReAdjustPos
 *
 *    Rev 1.0   16 Feb 1996 15:43:24   glennon
 * Initial revision.
 *
 */
#ifndef __WINDOWS_H
  #include <windows.h>
#endif

#ifdef INSTRUMENTED
  #include <stdio.h>
  #include <string.h>
  #include <ctype.h>
  #include "cc_inst.h"
#endif

#include "filter.hpp"

/////////////////////////////////////////////////////////////////////////////////
/* Closed caption decoder
    REQUIREMENTS
    1- decode at least 1 line of CC data
    2- accept input data in Y8 or 4:2:2 format
    3- indicate success and errors
    4- be callable from within an ISR
    5- DLL
    6- no defined error rates !

    DESIGN
    (refer to Video Demystified for info on CC waveform)
    The steps taken to decode the waveform are:

    1- Run a low pass Finite Impulse Response (FIR) filter over the input waveform.
    2- Determine the signal min & max.
    2- Characterize the run-in portion of signal. Derive the bitwidth from this characterization.
    3- Find the midpoint of the ramp to the start bit. Add 1/2 bit width to center on start bit.
    4- Scan through the rest of the buffer at bitwidth intervals to collect bits. Anything above
       the centerline is a 1, below the centerline is a 0.
    5- Convert results from 4 into characters.
    6- Check parity
*/
/////////////////////////////////////////////////////////////////////////////////

#ifdef _DEBUG
  #define PRIVATE public
  #define INLINE
#else
  #define PRIVATE private
  #define INLINE inline
#endif

//
// Following data is for use in FFT
//
#define FFT_SAMPLES   256
#define POWER     8    // 2 to the 8th power is 256 


class CCDecoder
{
  PRIVATE:
    bool isInitialized;                 // Figure it out
    int  m_hlen;                        // Total horiz length (in bytes) of a single signal
    int  m_samplesLen;                  // Length of sample buffer in bytes
    int  m_lines;                       // # of cc lines to decode
    BYTE far* m_pInputSamples;          // Ptr to the samples for the video card
    int  m_step;                        // Step size for sample data; 2 for 422
                                        //   1 for LUMA
    DWORD  m_bitWidth;                    // Calculated bit width.
    Filter theFilter ;
    double Real[FFT_SAMPLES] ;
    double Imag[FFT_SAMPLES] ;

  public:
    CCDecoder(int hlen=640,int maxlen=640);
    ~CCDecoder(void);
    int Init( int samplesLen, 
              int horizResolution, 
              DWORD decodeMode);
    int Decode( const BYTE far* samples,
                int   lines,
                const BYTE far* decodedData,
                int   far * decodedDataLen );

    // fft,filter etc
    void scale() ;
    void fft()   ;
    BOOL validRunin(BYTE * samples, int len, int midpoint) ;

  PRIVATE:
    BOOL ParityEven(char chr) ;

  #ifdef INSTRUMENTED
    int m_samplePoints[NUM_POINTS] ;
    int m_sampleNdx ;
    void OutputInstrumentation( FPCC_INST fpCCI);
    void DumpWaveForm(BYTE far * pDbg, WORD hlen);
  #endif
};
// EOF