/* File: CCEngine.cpp
 * Purpose: Contains code that captures and displays the closed captioning data
 * Author : sasha
 * $Log: CCEngine.cpp $
 * Revision 2.10  1998/04/02 21:36:51  dtharp
 * Merge DT_VIDEOHDR branch. Implements a new buffer 
 * management scheme.
 * Revision 2.9.1.1  1998/03/18 19:32:59  dtharp
 * Branch for passing videoheaders around
 * Revision 2.9  1997/10/23 16:54:48  dtharp
 * Fix for bug 838 : cc crash on gemini. Decoding is now done at 
 * display time instead of callback time.
 * Revision 1.1  1997/10/23 16:50:30  dtharp
 * Initial revision
 * Revision 2.8  1997/07/31 10:49:28  dtharp
 * Cleaned up multiple CC_CALLBACK definitions.
 * Revision 2.7  1997/06/27 16:57:38  dtharp
 * Callback now accepts an xtra argument (undefined at this time). Also
 * corrected possible buffer overrun condition.
 * Revision 2.6  1997/06/02 21:50:46  dtharp
 * Bug id 628: flashing pink box every 5 sec in preview. CC display
 * watchdog timer is now shut down when CC is paused.
 * Revision 2.5  1996/10/04 15:01:32  dtharp
 * Changes required for new CC Dll API names
 * Revision 2.4  1996/10/03 16:37:46  dtharp
 * Modified to use functions in api_cc.cpp rather than explicit pointers
 * into cc.dll.
 * Revision 2.3  1996/10/02 20:07:17  dtharp
 * Implemented modifications required to split the Closed Caption 
 * Decoder into a separate DLL.
 * Revision 2.2  1996/10/02 04:23:09  sasha
 * In Display() call function through the pointer now. So our own CC formatter
 * can be used, or external function can be used
 * Revision 2.1  1996/06/10 20:12:10  ssm
 * MarkBeta1AsRev2
 * Revision 1.7  1996/06/04 19:34:50  sasha
 * Changed CC channel members to be pointers. Changed StopDecode()
 * Added destructor
 * Revision 1.6  1996/06/04 01:02:30  sasha
 * Do not start if already running ( a routine check ). Deleted commented code
 * Revision 1.5  1996/05/31 17:42:23  sasha
 * Very close. Any minute now
 * Revision 1.4  1996/05/31 03:39:05  sasha
 * This seems to be the best so far
 * Revision 1.3  1996/05/30 23:11:53  sasha
 * Just next version.
 * Revision 1.2  1996/05/28 17:58:12  sasha
 * Checked in without saving first, duh
 * Revision 1.1  1996/05/28 17:52:13  sasha
 * Initial revision
 */
#include "ccengine.h"
#include "cc_api.h"
#include "dbgprint.h"

CCChannel *CCEngine::Even_;
CCChannel *CCEngine::Odd_ ;

BYTE      CCEngine::Decoded_ [CC_DECODE_BUF_SIZE];
volatile  int  CCEngine::DecodCnt_ = 0;
bool      CCEngine::Running_;
CC_CALLBACK CCEngine::Formatter_;

// The following defines a queue (fifo) to buffer
// incoming raw CC data.
  #define MAX_BUFFERS 10
  #define RAW_CC_BUF_SIZE 768 * 2 * 2
  char rawCCData[MAX_BUFFERS][RAW_CC_BUF_SIZE] ;
  int rawCcHead = 0 ;
  int rawCcTail = 0 ;

  #define INC_CC_PTR(a) (((a)+1) % MAX_BUFFERS)
  #define CC_Q_FULL     (INC_CC_PTR(rawCcHead) == rawCcTail)
  #define CC_Q_EMPTY    (rawCcHead == rawCcTail)
// end queue


/* Method: CCEngine::Init
 * Purpose: Initializes CC decode and display process
 */
void CCEngine::Init( HWND hWin )
{
   // initialize the display portion
   CC_InitDisplay( hWin );
   memset( Decoded_, '\0', sizeof( Decoded_ ) );
}

/* Method: CCEngine::StartDecode
 * Purpose: Starts CC decode process
 * Input: None
*/
void CCEngine::StartDecode()
{
   if ( Running_ )
      return;

   Odd_  = new CCChannel( VS_VBIO );
   if ( Odd_->OpenChannel() != Success ) {
      Odd_ = NULL;
      Running_ = false;
      return;
   }

   Even_ = new CCChannel( VS_VBIE );
   if ( Even_->OpenChannel() != Success ) {
      delete Odd_;
      Odd_ = NULL;
      delete Even_;
      Even_ = NULL;
      Running_ = false;
      return;
   }
   CC_InitDecode( 768 * 2 * 2, 768 * 2, BT819CC_IN_LUMA );

   Odd_->SetParentCallback( (CC_CALLBACK) Callback );
   Odd_->Start();
   Running_ = true;
}

/* Method: CCEngine::StopDecode
 * Purpose: Shuts CC decode down
 * Input: None
*/
void CCEngine::StopDecode()
{
   if ( Running_ )
      Odd_->Stop();

   delete Odd_;
   delete Even_;

   Odd_ = NULL;
   Even_ = NULL;
   
   // Turn off the CC display watchdog...
   CC_KillWatchdog( );

   Running_ = false;
}


/* Method: CCEngine::Display
 * Purpose: Displays a CC characters   (called via a timer )
 * Input: None
*/
void CCEngine::Display()
{
   if ( !Running_ )
      return;

   // The following while loop handles the decoding of all raw cc buffers
   // queued during callbacks. The decode was moved here because it took 
   // too much time during the callback, causing problems.
   while(!CC_Q_EMPTY)
   {
     int Len = 2;
     int ret;
     if ( ( ret = CC_Decode( rawCCData[rawCcTail], 2, Decoded_ + DecodCnt_, &Len ) ) == BT819CC_ERR_NONE ) 
     {
        if (DecodCnt_ + Len < CC_DECODE_BUF_SIZE - 2)
           DecodCnt_ += Len;
        else
           DebugPrint((0,"CC Decode Buffer is full\n")) ;

        //DecodCnt_ %= CC_DECODE_BUF_SIZE;
     } else
        if ( ret == BT819CC_ERR_PARITY )
           D1( "parity " );
        else
           D1( "no data " );

     rawCcTail = INC_CC_PTR(rawCcTail) ;
   }

   int OldCnt = DecodCnt_;
   DecodCnt_ -= OldCnt;
   if ( OldCnt )      
      Formatter_( Decoded_, OldCnt );
}

/* Method: CCEngine::Callback
 * Purpose: This function is called in response to the VBI interrupt
 * Input: dwAddr: DWORD - address of the data buffer accosiated with the interrupt
 */
void _loadds CALLBACK CCEngine::Callback( DWORD dwAddr, DWORD )
{
   LPBYTE pTmp;

     if ( dwAddr == Odd_->Buf1Base_ )
        pTmp = Odd_->VideoHdr1_->lpData;
     else {
        if ( dwAddr == Odd_->Buf2Base_ )
           pTmp = Odd_->VideoHdr2_->lpData;
        else 
           return;
     }
     pTmp += 768 * 2 * 10;

     if (!CC_Q_FULL) 
     {
       memcpy(&rawCCData[rawCcHead],pTmp,RAW_CC_BUF_SIZE) ;
       rawCcHead = INC_CC_PTR(rawCcHead) ;
     }
     else
       DebugPrint((0,"Cc Queue is Full!")) ;

}

/* Destructor: CCEngine::~CCEngine
 * Purpose: Disposes of the CCEngine
 */
CCEngine::~CCEngine()
{
   StopDecode();
}
