/*
   File: Regbase.h
   Purpose: Contains definitions of the RegBase class
   Author: sasha
 * $Log: regbase.h $
 * Revision 2.2  1997/05/14 17:45:16  dtharp
 * Added #undef FAR to avoid compiler warnings.
 * Revision 2.1  1996/06/10 20:13:46  ssm
 * MarkBeta1AsRev2
 * Revision 1.17  1996/04/25 23:49:08  sasha
 * Added mytypes.h. Always include windows.h
 * Revision 1.16  1996/02/09 00:51:00  sasha
 * Changed GetBaseAddress() to be static
 * Revision 1.15  1996/01/04 22:51:23  sasha
 * Changed m_szName to szRegName_ to reflect used naming convention
 * Put code to properly initialize szRegName_. Some formatting change
 * Revision 1.14  1995/12/22 01:45:30  ssm
 * changed char* varname to char *varname
 * Revision 1.13  1995/12/08 21:23:15  sasha
 * Added RR type of registers
 * Revision 1.12  1995/12/08 20:58:09  sasha
 * 'Unfixed' dbg.h. Moved #include <string.h> to the non-VxD branch of compilation
 * Revision 1.11  1995/12/08 20:21:56  ssm
 * replaced hard tabs with 3 spaces
 * Revision 1.10  1995/11/29 00:11:48  ssm
 * fixed "debug" ifdef;  added m_szName
 * Revision 1.9  1995/11/22 18:16:01  sasha
 * Revision 1.8  1995/09/07 23:36:26  sasha
 * Included vtoolsd headers
 * Revision 1.7  1995/07/29 01:36:02  SASHA
 * Added virtual destructor to the RegBase class.
 * Added description of the dwShadow_ member
 * Added descriptions of the RW, RO, WO
 * Added comments to the Set/GetShadow methods
 * Revision 1.6  1995/07/21 23:23:56  SASHA
 * Revision for the code review. Should be no changes from previous check-in.
 * Revision 1.5  1995/07/21 22:31:34  SASHA
 * Changed the return type of GetBaseAddress from DWORD to PBYTE,
 * so there will be no need for casts in the source files
 * Revision 1.4  1995/07/18 01:10:58  SASHA
 * Added private default constructor to the RegBase class.
 * Revision 1.3  1995/07/18 00:31:48  SASHA
 * Added dwShadow_ member; added SetShadow() and GetShadow() 
 * functions to the class. Removed the private default constructor
*/                  
#ifndef __REGBASE_H
#define __REGBASE_H

#ifdef VXD
   #include <vtoolscp.h>
   #undef FAR
#else
   #include <string.h>
#endif

#ifndef __WINDOWS_H
#include <windows.h>
#endif

#ifndef __DBG_H
#include "dbg.h"
#endif

#include "mytypes.h"

/* Type: AllFs
 * Purpose: To be used as an error return value from register accessing
 *   functions. All bits are set to 1.
 */
const DWORD AllFs = ~0L;

/* Function: ReturnAllFs
 * Purpose: This function is used in the register access methods to indicate
 *   that some sort of error has occured. Used for easing the debugging, as
 *   it contains a macro to print the error if DEBUG is #defined
 */
inline DWORD ReturnAllFs()
{
    OUTPUT_MESS( ALLFS );
    return  AllFs;
}

/*
 * Type: RegisterType
 * Purpose: A type to differentiate between diferent kinds of registers.
 *   Depending on the type register may not peforms certain operations
 *   RW - read/write, RO - read-only, WO - write-only
*/
typedef enum { RW, RO, WO, RR } RegisterType;

/* Class: RegBase
 * Purpose:
 *   Defines the interface and encapsulates the register access.
 * Attributes:
 *   pBaseAddress_: DWORD, static. Holds the base address of the registers. On the
 *   PCI bus it is a 32 bit memory address. On the ISA bus it is a 16 bit I/O address.
 *   type_: RegisterType - defines the access permission for the register.
 *   dwShadow_: DWORD - a local copy of the register. Used for returning a value
 *   of write-only registers
 * Operations:
 *   operator DWORD(): data access method. Pure virtual
 *   DWORD operator=( DWORD ): assignment operator. Pure virtual. This assignment
 *      operator does not return a reference to the class because of the performance reasons
 *   void SetBaseAddress( DWORD )
 *   DWORD GetBaseAddress()
 *   RegisterType GetRegisterType()
 *   void SetShadow( DWORD ): assigns a value of a register to a shadow
 *   DWORD GetShadow( void ): retrieves a value from a shadow
 */
class RegBase
{
    private:
         static DWORD dwBaseAddress_;
         RegisterType type_;
         DWORD        dwShadow_;
         char         szRegName_ [17];

         RegBase();

    protected:
         void  SetShadow( DWORD dwValue );
         DWORD GetShadow();
    public:
         RegBase( RegisterType aType, const char *szName = NULL ) :
            type_( aType ), dwShadow_( 0 )
         {
            *szRegName_ = '\0';
            if ( szName ) strncpy( szRegName_, szName, 16 );
         }

         void GetRegisterName( char *szName )
         { strcpy( szName, szRegName_ ); }

         static void SetBaseAddress( DWORD dwBase ) { dwBaseAddress_ = dwBase; }
         static LPBYTE GetBaseAddress() { return (LPBYTE)dwBaseAddress_; }
         RegisterType GetRegisterType() { return type_; }
         virtual operator DWORD() = 0;
         virtual DWORD operator=( DWORD dwValue ) = 0;
         virtual ~RegBase() {}
};

/* Method: RegBase::SetShadow
 * Purpose: Used to store the value of a register in the shadow
 * Input: dwValue: DWORD - new value of a register
 * Output: None
 * Note: inline
 */
inline void  RegBase::SetShadow( DWORD dwValue ) { dwShadow_ = dwValue; }

/* Method: RegBase::GetShadow
 * Purpose: Used to obtain the last value written to a write-only register
 *    from the shadow
 * Input: None
 * Output: DWORD
 * Note: inline
 */
inline DWORD RegBase::GetShadow() { return dwShadow_; }

#endif __REGBASE_H
