/* File: Risceng.cpp
 * Purpose: Contains implementation of the RISCEng class
 * Author: sasha
 * $Log: risceng.cpp $
 * Revision 2.5  1998/04/02 21:43:33  dtharp
 * Merge DT_VIDEOHDR branch. Implements a new buffer 
 * management scheme.
 * Revision 2.4.2.1  1998/03/18 19:37:09  dtharp
 * Branch for passing videoheaders around
 * Revision 2.4  1997/02/19 21:19:20  dtharp
 * Grafted DT_PAL_CAPTURE_RATE branch (bug id 384)
 * Revision 2.3.2.1  1997/02/10 21:51:08  dtharp
 * Branch for bug #384. Added video format parameter.
 * Revision 2.3  1996/09/27 00:00:26  sasha
 * Bug ID# 254. Added bool parameter to the CreateProgram() to indicate
 * that resyncs need to be inserted.
 * Revision 2.2  1996/09/22 23:32:03  sasha
 * Added DWORD argument to the CreateProgram() to implement the 
 * functionality needed for the 640x480 YUV AVI capture.
 * Revision 2.1  1996/06/10 20:13:55  ssm
 * MarkBeta1AsRev2
 * Revision 1.7  1996/01/06 02:07:03  sasha
 * Changed ColorFormat to ColFmt
 * Revision 1.6  1995/11/02 20:57:06  sasha
 * First major check-in for testing
 * Revision 1.5  1995/10/18 05:55:30  sasha
 * Changed SIZE to TSize. Removed first_ from the class. 
 * Changed Skip function
 * Revision 1.4  1995/10/15 06:39:20  sasha
 * Added support for clipping.  
 * Revision 1.3  1995/10/12 22:49:12  sasha
 * Added first_ member to the class and use it in the call to RISCProgram::Create
 * Revision 1.2  1995/10/12 18:24:44  sasha
 * Changed the skipping a bit
 * Revision 1.1  1995/10/04 04:05:18  sasha
 * Initial revision
 */

#include "risceng.h"


/* Method: RISCEng::CreateProgram
 * Purpose: Creates a first RISC program for a stream (field)
 * Input: aField: VidField - defines what field the program is for
 *   ImageSize: SIZE & - defines dimentions of the image
 *   dwPitch: DWORD - pitch to be used for data buffer . Useful for producing
 *   interlaced images.
 *   aFormat: ColorFormat - defines type of data
 *   dwBufAddr: DWORD - address of the destination buffer
 * Output: RiscPrgHandle
 */
RiscPrgHandle RISCEng::CreateProgram( TSize &ImageSize, DWORD dwPitch, DWORD dwOffset,
   ColFmt aFormat, LPVIDEOHDR lpVH, VideoFormat vf, bool Intr, 
   DWORD dwPlanrAdjust, bool rsync )
{
   // create yet another risc program object
   RISCProgram *YAProgram = new RISCProgram( ImageSize, dwPitch, dwOffset, aFormat, vf );

   // and make it create the program itself
   if ( YAProgram->Create( Intr, lpVH, dwPlanrAdjust, rsync ) != Success ) {
      delete YAProgram;
      YAProgram = NULL;
   }

   return YAProgram;
}

/* Method: RISCEng::AddProgram
 * Purpose: This function is used to implement double buffering
 * Input: parent: RiscPrgHandle - handle of a program to add new program to
 *   dwBufAddr: DWORD - address of the data buffer
 * Output:
 * Note: Maybe should pass field instead of program handle. That way nobody
 *   outside this class has to know any details and maintain records.
 */
RiscPrgHandle RISCEng::AddProgram( RiscPrgHandle parent, LPVIDEOHDR lpVH )
{
   RiscPrgHandle NewProg = CreateProgram( parent->ImageSize_,                      // Size
                                          parent->dwBufPitch_,                     // pitch
                                          parent->dwOffset_,                       // offset. Zero unless this is an interleaved frame
                                          parent->BufFormat_.GetColorFormat(),     // color format
                                          lpVH ,                                   // pointer to videohdr containing buffer to fill
                                          parent->vidFormat_);                     // 
   Chain( parent, NewProg );
   return NewProg;
}

/* Method: RISCEng::DestroyProgram
 * Purpose: Removes a program from a chain and destroy it
 * Input: ToDie: RiscPrgHandle - pointer to a program to destroy
 * Output: None
 */
void RISCEng::DestroyProgram( RiscPrgHandle ToDie )
{
   // take care of updating links
   Skip( ToDie );

   delete ToDie;
}

/* Method: RISCEng::SetClipping
 * Purpose:
 * Input:
 * Output:
 */
void RISCEng::SetClipping( RiscPrgHandle hProg, const RGNDATA &const ClipList )
{
   hProg->SetClipping( (LPRECT)ClipList.Buffer, ClipList.rdh.nCount );
}

/* Method: RISCEng::ChangeAddress
 * Purpose:
 * Input:
 * Output:
 */
void RISCEng::ChangeAddress( RiscPrgHandle prog, LPVIDEOHDR lpNewVH )
{
   prog->ChangeAddress( lpNewVH );
}

/* Method: RISCEng::Chain
 * Purpose: Chains two RISC programs together
 * Input: hParent: RiscPrgHandle - pointer in reality
 *   hChild: RiscPrgHandle - pointer in reality
 * Output: None
 */
void RISCEng::Chain( RiscPrgHandle hParent, RiscPrgHandle hChild )
{
   hParent->SetChain( hChild );
}

/* Method: RISCEng::Skip
 * Purpose: Sets the given program so it is bypassed by DMA
 * Input: hToSkip: RiscPrgHandle - pointer in reality
 * Output: None
 * Note: The way things are right now a program will always have a child and a
 *   parent, even if it is its own parent and child
 */
void RISCEng::Skip( RiscPrgHandle pToSkip )
{
//   RiscPrgHandle pParent = pToSkip->GetParent();
   pToSkip->Skip();
/*
   if ( pParent )
      pToSkip->Skip();
   else  // can not skip as have nowhere to skip
      pToSkip->CreateLoop( true );
*/
}

