/* File: PsPnP.cpp
 * Purpose: Contains PnP related code
 * Author: sasha
 * $Log: PsPnP.cpp $
 * Revision 2.8  1998/04/21 00:54:19  dtharp
 * No functional changes--Changed debug level on numerous
 * debug prints
 * Revision 2.7  1998/04/17 22:56:11  dtharp
 * Protected from bogus devnode notification during CM time
 * Revision 2.6  1998/03/10 01:00:09  dtharp
 * Merged in HP remove fix. CONFIG_PREREMOVE2 message is now
 * handled.
 * Revision 2.5  1998/02/27 18:18:29  dtharp
 * Capture Controler constructor now takes devID as a param.
 * Revision 2.4  1998/01/17 00:58:18  rudyl
 * Implemented software work-around for BANDGAP start-up yield problem.
 * Revision 2.3  1997/07/22 15:36:19  dtharp
 * Added FNGetPciHdr functionality via DevIOCtl and VxD Service.
 * Revision 2.2  1997/04/02 00:18:52  dtharp
 * Grafted ReadConfig branch (driver reads and stores DevId and 
 * Revision info from PCI config space).
 * Revision 2.1.2.1  1997/03/31 18:14:55  dtharp
 * Installed code to read RevId and DevId from config space.
 * Revision 2.1  1996/06/10 20:12:46  ssm
 * MarkBeta1AsRev2
 */

#include "device.h"
#include "dbgPrint.h"



/* Method: PsDevice::ConfigHandler
 * Purpose: Handles PnP config messages
 * Input: cfFuncName: CONFIGFUNC, PnP function code
 *   scfSubFuncName: SUBCONFIGFUNC, PnP sub-function code
 *   dnToDevNode: DEVNODE, device node
 *   dwRefData: DWORD, reference data
 *   ulFlags: ULONG
 * Output: CONFIGRET
 */
CONFIGRET CM_HANDLER PsDevice::ConfigHandler( CONFIGFUNC cfFuncName,
   SUBCONFIGFUNC scfSubFuncName, DEVNODE dnToDevNode, DWORD dwRefData,
   ULONG ulFlags )
{
   switch ( cfFuncName ) {
   case CONFIG_START:

      DebugPrint((0,"CONFIG_START : subFunc %s, devnode %x, RefData %x, flags %x\n", 
         scfSubFuncName == CONFIG_START_DYNAMIC_START ? "CONFIG_START_DYNAMIC_START" :
           scfSubFuncName == CONFIG_START_FIRST_START ? "CONFIG_START_FIRST_START" : "Unknown subfunc",
         dnToDevNode,
         dwRefData,
         ulFlags
         )) ;

      if (scfSubFuncName == CONFIG_START_DYNAMIC_START) {
         DebugPrint((0,"Ignoring CONFIG_START_DYNAMIC_START\n")) ;
      }
      else if (scfSubFuncName == CONFIG_START_FIRST_START) {
         // You may or may not treat CONFIG_START_FIRST_START
         // different from CONFIG_START_DYNAMIC_START.

         // As far as CM goes, the first time it would
         // have called with CONFIG_START_DYNAMIC_START,
         // it will call with CONFIG_START_FIRST_START
         // instead.
         CMCONFIG ccBuffer;

         _CONFIGMG_Get_Alloc_Log_Conf( &ccBuffer, dnToDevNode,
            CM_GET_ALLOC_LOG_CONF_ALLOC );

         int i ;
         DebugPrint((1,"Got CM info for devnode %x:\n",dnToDevNode)) ;
         DebugPrint((1,"  wNumMemWindows %d\n",ccBuffer.wNumMemWindows));      // Num memory windows
         for ( i = 0 ; i < ccBuffer.wNumMemWindows; i++) {
           DebugPrint((1,"    dMemBase[%d] = %x\n",i,ccBuffer.dMemBase[i]));  // Memory window base
           DebugPrint((1,"    dMemLength[%d] = %x\n",i,ccBuffer.dMemLength[i]));  // Memory window length
           DebugPrint((1,"    wMemAttrib[%d] = %x\n",i,ccBuffer.wMemAttrib[i]));  // Memory window Attrib
         }
         DebugPrint((1,"  wNumIOPorts %d\n",ccBuffer.wNumIOPorts));      // Num IO ports
         for ( i = 0 ; i < ccBuffer.wNumIOPorts; i++) {
           DebugPrint((1,"    wIOPortBase[%d] = %x\n",i,ccBuffer.wIOPortBase[i]));  // I/O port base
           DebugPrint((1,"    wIOPortLength[%d] = %x\n",i,ccBuffer.wIOPortLength[i]));  // I/O port length
         }
         DebugPrint((1,"  wNumIRQs %d\n",ccBuffer.wNumIRQs));      // Num IRQ info
         for ( i = 0 ; i < ccBuffer.wNumIRQs; i++) {
           DebugPrint((1,"    bIRQRegisters[%d] = %x\n",i,ccBuffer.bIRQRegisters[i]));  // IRQ list
           DebugPrint((1,"    bIRQAttrib[%d] = %x\n",i,ccBuffer.bIRQAttrib[i]));    // IRQ Attrib list
         }
         DebugPrint((1,"  wNumDMAs %d\n",ccBuffer.wNumDMAs));      // Num DMA channels
         for ( i = 0 ; i < ccBuffer.wNumDMAs; i++) {
           DebugPrint((1,"    bDMALst[%d] = %x\n",i,ccBuffer.bDMALst[i]));  // DMA list
           DebugPrint((1,"    wDMAAttrib[%d] = %x\n",i,ccBuffer.wDMAAttrib[i]));  // DMA Attrib list
         }

         // For some reason , on some systems, we're getting called with some bogus devnode.
         // So we need to do a sanity check on the devnode before we assign resources:
         if (ccBuffer.wNumMemWindows == 0 || ccBuffer.wNumIRQs == 0)
         {
            // must not be us!
            DebugPrint((1,"*** Detected Bogus Devnode\n")) ;
            return CR_SUCCESS;
//            return CR_FAILURE;
         }

         RegBase::SetBaseAddress(
            (DWORD)MapPhysToLinear( (const void *)ccBuffer.dMemBase [0], 0xFFF, 0 ) );

         if (HWInt) {
            DebugPrint((1,"Unhooking and deleteing existing HWInt\n")) ;
            HWInt->unhook() ;
            delete HWInt;
         }

         DebugPrint((1,"Instantiating PsInterrupt\n")) ;
         HWInt = new PsInterrupt( ccBuffer.bIRQRegisters [0] );
         DebugPrint((1,"Done Instantiating PsInterrupt: HWInt = %x IRQ %d\n",HWInt,ccBuffer.bIRQRegisters [0])) ;
         if ( !HWInt->hook() ){
            DebugPrint((1,"HWInt->hook failed! returning CR_FAILURE from ConfigHandler\n")) ;
            return CR_FAILURE;
         }
         DebugPrint((1,"calling HWInt-physicalUnmask()\n")) ;
         HWInt->physicalUnmask();


         // Read PCI Config space to determine the device config. For now, we're only interested
         // in the device ID and the revision.
         // Here's the breakdown:
         //                         PCI DevId 
         // Bt848KPF  / 848 Die     848
         // Bt848KPF  / 849 Die     848
         // Bt848AKPF / 849 Die     848
         // Bt849AKPF / 849 Die     849   *** Bond Option
         // 
         //                         PCI Rev
         // 848 Die Rev A           00   
         // 848 Die Rev B           01   
         // 848 Die Rev C           11   *** Production Bt848
         // 849 Die Rev A           02   
         // 849 Die Rev B           12   *** Production Bt848,848A,849A

         CONFIGMG_Call_Enumerator_Function(dnToDevNode, 
                                           0,                      // PCI_ENUM_FUNC_GET_DEVICE_INFO, 
                                           0,                      // ULONG ulRefData, (byte index of rev)
                                           &pciCfgHdr_,                // PFARVOID pBuffer, 
                                           sizeof(pciCfgHdr_),         // ULONG ulBufferSize, 
                                           0) ;                    // ULONG ulFlags

         GetPsDevice()->CreateCaptureController(pciCfgHdr_.DeviceID);

         DebugPrint((0,"  VendorID %d  0x%x\n",pciCfgHdr_.VendorID,pciCfgHdr_.VendorID ))     ;
         DebugPrint((0,"  DeviceID %d  0x%x\n",pciCfgHdr_.DeviceID,pciCfgHdr_.DeviceID ))     ;
         DebugPrint((0,"  Command %d  0x%x\n",pciCfgHdr_.Command,pciCfgHdr_.Command  ))     ;
         DebugPrint((0,"  Status %d  0x%x\n",pciCfgHdr_.Status,pciCfgHdr_.Status   ))     ;
         DebugPrint((0,"  RevisionID %d  0x%x\n",pciCfgHdr_.RevisionID,pciCfgHdr_.RevisionID ))   ;
         DebugPrint((0,"  ClassCode %d  0x%x\n",pciCfgHdr_.ClassCode,pciCfgHdr_.ClassCode  ))   ;
         DebugPrint((0,"  LatencyTimer %d  0x%x\n",pciCfgHdr_.LatencyTimer,pciCfgHdr_.LatencyTimer )) ;
         DebugPrint((0,"  HeaderType0 %d  0x%x\n",pciCfgHdr_.HeaderType0,pciCfgHdr_.HeaderType0 ))  ;
         DebugPrint((0,"  BaseAddr0 %d  0x%x\n",pciCfgHdr_.BaseAddr0,pciCfgHdr_.BaseAddr0 ))    ;
         DebugPrint((0,"  InterruptLine %d  0x%x\n",pciCfgHdr_.InterruptLine,pciCfgHdr_.InterruptLine ));
         DebugPrint((0,"  InterruptPin %d  0x%x\n",pciCfgHdr_.InterruptPin,pciCfgHdr_.InterruptPin )) ;
         DebugPrint((0,"  Min_Gnt %d  0x%x\n",pciCfgHdr_.Min_Gnt,pciCfgHdr_.Min_Gnt ))      ;
         DebugPrint((0,"  Max_Lat %d  0x%x\n",pciCfgHdr_.Max_Lat,pciCfgHdr_.Max_Lat ))      ;
         
         // Software work-around for BANDGAP start-up yield problem:
         // Register XXX bit 7 powers up low. it must be turned on - a one -
         // then reset back to a low in order to properly start-up the bandgap. 
         {      
            const Reg_XXX_Offset = 0x058;

            DWORD pRegXXX = DWORD( RegBase::GetBaseAddress() + Reg_XXX_Offset );
            DWORD regXXX = *(PDWORD)pRegXXX;
            DebugPrint((0,"BANDGAP: BaseAddr=%X regXXXAddr=%X regXXX=%X\n", RegBase::GetBaseAddress(), pRegXXX, *(PDWORD)pRegXXX)) ;
            regXXX |= 0x80;   // turn on bit 7
            *(PDWORD)pRegXXX = regXXX;
            DebugPrint((0,"BANDGAP: BaseAddr=%X regXXXAddr=%X regXXX=%X\n", RegBase::GetBaseAddress(), pRegXXX, *(PDWORD)pRegXXX)) ;
            regXXX &= ~0x80;  // turn off bit 7
            *(PDWORD)pRegXXX = regXXX;
            DebugPrint((0,"BANDGAP: BaseAddr=%X regXXXAddr=%X regXXX=%X\n", RegBase::GetBaseAddress(), pRegXXX, *(PDWORD)pRegXXX)) ;
         }
      }
      else
         DebugPrint((0,"Bad sub func\n")) ;
      return CR_SUCCESS;

   case CONFIG_TEST:
      DebugPrint((0,"CONFIG_TEST\n")) ;
      return CR_SUCCESS;

   case CONFIG_STOP:
      DebugPrint((0,"CONFIG_STOP : %s\n", 
         scfSubFuncName == CONFIG_STOP_DYNAMIC_STOP ? "CONFIG_STOP_DYNAMIC_STOP" :
           scfSubFuncName == CONFIG_STOP_HAS_PROBLEM ? "CONFIG_STOP_HAS_PROBLEM" : "Unknown subfunc"
         )) ;
      return CR_SUCCESS;

   case CONFIG_REMOVE:
   case CONFIG_PREREMOVE2:                          // Devnode must stop using config.                       
      DebugPrint((0, "%sCONFIG_REMOVE  (scfSubFuncName %x)\n",
                   cfFuncName == CONFIG_REMOVE? "CONFIG_REMOVE" : "CONFIG_PREREMOVE2",
                   scfSubFuncName ));
      // deallocate per instance data.
      if (HWInt) {
         HWInt->unhook();
         delete HWInt ;
         HWInt = NULL ;
      }
      return CR_SUCCESS;

   default:
      DebugPrint((0, "PNP: Default: " ));

      switch(cfFuncName)
      {
      case CONFIG_FILTER:                              // Ancestors must filter requirements.
         DebugPrint((0,"CONFIG_FILTER\n")) ;           
         break ;                                       
      case CONFIG_START:                               // Devnode dynamic initialization.
         DebugPrint((0,"CONFIG_START\n")) ;            
         break ;                                       
      case CONFIG_STOP:                                // Devnode must stop using config.
         DebugPrint((0,"CONFIG_STOP\n")) ;             
         break ;                                       
      case CONFIG_TEST:                                // Can devnode change state now.
         DebugPrint((0,"CONFIG_TEST\n")) ;             
         break ;                                       
      case CONFIG_REMOVE:                              // Devnode must stop using config.
         DebugPrint((0,"CONFIG_REMOVE\n")) ;           
         break ;                                       
      case CONFIG_ENUMERATE:                           // Devnode must enumerated.
         DebugPrint((0,"CONFIG_ENUMERATE\n")) ;        
         break ;                                       
      case CONFIG_SETUP:                               // Devnode should download driver.
         DebugPrint((0,"CONFIG_SETUP\n")) ;            
         break ;                                       
      case CONFIG_CALLBACK:                            // Devnode is being called back.
         DebugPrint((0,"CONFIG_CALLBACK\n")) ;         
         break ;
      case CONFIG_APM:                                 // APM functions.
         DebugPrint((0,"CONFIG_APM\n")) ;              
         break ;
      case CONFIG_TEST_FAILED:                         // Continue as before after a TEST.
         DebugPrint((0,"CONFIG_TEST_FAILED\n")) ;                             
         break ;                                                              
      case CONFIG_TEST_SUCCEEDED:                      // Prepare for the STOP/REMOVE.                       
         DebugPrint((0,"CONFIG_TEST_SUCCEEDED\n")) ;                          
         break ;                                                              
      case CONFIG_VERIFY_DEVICE	:                      // Insure the legacy card is there.                       
         DebugPrint((0,"CONFIG_VERIFY_DEVICE	\n")) ;                         
         break ;                                                              
      case CONFIG_PREREMOVE:                           // Devnode must stop using config.                       
         DebugPrint((0,"CONFIG_PREREMOVE (scfSubFuncName %x)\n",scfSubFuncName)) ;                               
         return CR_SUCCESS;
                                                                       
      case CONFIG_SHUTDOWN:                            // We are shutting down.                       
         DebugPrint((0,"CONFIG_SHUTDOWN\n")) ;                                
         break ;                                                              
                                                                       
      case CONFIG_READY:                               // The devnode has been setup.
         DebugPrint((0,"CONFIG_READY\n")) ;                   
         break ;                                              
      case CONFIG_PROP_CHANGE:                         // The property page is exiting.       
         DebugPrint((0,"CONFIG_PROP_CHANGE\n")) ;             
         break ;                                              
      case CONFIG_PRIVATE:                             // Someone called Call_Handler.       
         DebugPrint((0,"CONFIG_PRIVATE\n")) ;                 
         break ;                                              
      case CONFIG_PRESHUTDOWN:                         // We are shutting down       
         DebugPrint((0,"CONFIG_PRESHUTDOWN\n")) ;             
         break ;                                              
      default:                                                
         DebugPrint((0,"**UNKNOWN MSG** \n")) ;               
         break;                                                     
      }                                                             
      return CR_DEFAULT;                                            
	}                                                                 
}                                                                   
                                                                    
/* Method: PsDevice::OnPnpNewDevnode
 * Purpose: Handles PnP notifications
 * Input: DevNode: DWORD, device node handle
 *   LoadType: DWORD
 * Output: CONFIGRET
 */
CONFIGRET PsDevice::OnPnpNewDevnode( DWORD DevNode, DWORD LoadType )
{
   DebugPrint((0, "pnp\n" ));

   switch ( LoadType ) {
   case DLVXD_LOAD_DRIVER:
   	_CONFIGMG_Register_Device_Driver( DevNode, ConfigHandler, (DWORD)this, 0 );
      break;
   default:
      return CR_DEFAULT;
   }
   return CR_SUCCESS;
}

