/* File: Cliplist.cpp
 * Purpose: Clip / View List merge code
 * $Log: cliplist.cpp $
 * Revision 1.7  1998/04/02 21:39:38  dtharp
 * Merge DT_VIDEOHDR branch. Implements a new buffer 
 * management scheme.
 * Revision 1.6.2.1  1998/03/27 15:28:28  dtharp
 * Branch for passing videohdrs around.
 * Revision 1.6  1997/06/10 17:58:51  dtharp
 * Added some protections for overunning the end of buffers
 * Revision 1.5  1997/06/01 17:06:47  dtharp
 * CombineLists now takes currCombinedList parameter.  This reduces
 * number of memory allocations. Also added several DebugAsserts().
 * Revision 1.4  1997/05/14 17:18:13  dtharp
 * Addressed minor compiler warnings.
 * Revision 1.3  1997/04/27 19:04:20  dtharp
 * Added fcn DumpList().
 * Revision 1.2  1997/04/25 16:32:30  dtharp
 * Added header
*/

#include "cliplist.h"

#include "dbgprint.h"

void DumpList(LPSTR 
#ifdef DEBUG
                    listName
#endif
                    , RGNDATA * pRgnData)
{
  if (pRgnData)
  {
     DebugPrint((0,"--------- %s ----------\n"
                   //"  pointer: %08lx\n" 
                   //"  Bounding Rect: %ld %ld %ld %ld\n"
                   "  Count: %d\n", 
                   listName,
                  // pRgnData,
                  // pRgnData->rdh.rcBound.left,
                  // pRgnData->rdh.rcBound.top,
                  // pRgnData->rdh.rcBound.right,
                  // pRgnData->rdh.rcBound.bottom,
                   pRgnData->rdh.nCount)) ;
#ifdef DEBUG
     LPRECT pRects = (LPRECT) &pRgnData->Buffer[0]  ;
#endif
     for (DWORD i = 0 ; i < pRgnData->rdh.nCount && i < 100UL; i++)
     {
         DebugPrint((0,"      %d %d %d %d\n",
                       pRects[i].left,
                       pRects[i].top,
                       pRects[i].right,
                       pRects[i].bottom)) ;
     }
  }
}

//
// Helper function for combining clip/view lists
//

static DWORD VertSplit (RECT * r, DWORD numRects, LONG xCoord)
{
   DWORD nSplit = 0 ;
   for (DWORD i = 0 ; i < numRects ; i++)
   {
      if (xCoord > r[i].left && xCoord < r[i].right)
      {
         r[numRects+nSplit] = r[i] ;
         r[numRects+nSplit].left = r[i].right = xCoord ;
         nSplit++ ;
      }
   }
   return numRects+nSplit ;
}

static DWORD HorizSplit (RECT * r, DWORD numRects, LONG yCoord)
{
   DWORD nSplit = 0 ;
   for (DWORD i = 0 ; i < numRects ; i++)
   {
      if (yCoord > r[i].top && yCoord < r[i].bottom)
      {
         r[numRects+nSplit] = r[i] ;

         r[numRects+nSplit].top = r[i].bottom = yCoord ;
         nSplit++ ;
      }
   }
   return numRects+nSplit ;
}

// must return true if rPiece is a subrectangle of rSub.
// Cases include: 
//       1. rPiece == rSub
//       2. rPiece has all verticies inside rSub
//       3. rPiece is a common subrectangle (shares a side or vertex)
static BOOL isSubRectangle(RECT & rPiece, RECT & rSub)
{
   BOOL bRet = FALSE ;

   if (rPiece.left >= rSub.left &&
       rPiece.right <= rSub.right &&
       rPiece.top >= rSub.top &&
       rPiece.bottom <= rSub.bottom)
       bRet = TRUE ;

   return bRet ;
}

//
// rTarget is an array of 10 rectangles (9 being the maximum number
// of rectangles we will split into) initialized with the first element
// set to the target rectangle.
//
// returns number of resultant rectangles. This ranges from 0 to 8.
//
// No effort is made to coalesce adjacent rectangles here. This 
// may need to change...
//
static DWORD SubtractRect(RECT * rTarget, RECT & rSub)
{
   DWORD i,j ;
   //
   // Split the target rectangle based on rSub. This can result in 
   // in up to 9 subrectangles.
   //
   DWORD nRects = 1 ;
   nRects = VertSplit(rTarget,nRects,rSub.left) ;
   nRects = VertSplit(rTarget,nRects,rSub.right) ;
   nRects = HorizSplit(rTarget,nRects,rSub.top) ;
   nRects = HorizSplit(rTarget,nRects,rSub.bottom) ;

   //
   // Now iterate through the rectangles and see if any are 
   // subrectangles of rSub.
   //
   for (i = nRects-1 ; (int) i >= 0 ; i--)
   {
      if (isSubRectangle (rTarget[i],rSub))
      {
         // if we have a subRectangle, we want to remove
         // it from the list.
         for (j = i ; j < nRects ; j++)
            rTarget[j] = rTarget[j+1] ;

         nRects-- ;
      }
   }

   //
   // Coalesce adjacent horizontal rectangles 
   //
   // A rectangle is adjacent horizontally if the top and bottom are equal
   // and the right value of one rect equals the left value of another.
   // 
   // We can assume that the rectangles are ordered in the array, i.e.
   // that adjacent rects are sequential in the array
   #define isAdjacent(a,b) \
     ((a).top == (b).top && (a).bottom == (b).bottom && (a).right == (b).left)

   for (i = 0 ; i < nRects ; i++)
   {
      while (isAdjacent(rTarget[i],rTarget[i+1]))
      {
         rTarget[i].right = rTarget[i+1].right ;
         for (j = i+1 ; j < nRects; j++)
         {
            rTarget[j] = rTarget[j+1] ;
         }
         nRects-- ;
      }
   }

   return nRects ;
}

RGNDATA * CombineLists(RGNDATA * currCombinedList, RGNDATA & viewList, RGNDATA & clipList)
{
   RECT * pClipRects = (RECT *) clipList.Buffer ; 

   // allocate worst case return region data (avoid doing this in loop)
   DWORD maxRects = 1000UL ;
   #define ALLOC_SIZE (sizeof(RGNDATAHEADER) + (maxRects * sizeof(RECT)))
   RGNDATA * pRet = currCombinedList ? currCombinedList : (RGNDATA *) new char [ALLOC_SIZE];
   DebugAssert((bool(pRet != NULL),"Bad Alloc\n")) ;

   if (!pRet)
      return NULL ;

   BYTE * p = (BYTE *) pRet ;
   p[ALLOC_SIZE -1] = 0xa5 ; // sentinel

   pRet->rdh = viewList.rdh ;                   // copy header info
   pRet->rdh.nCount = 0 ;                       
   RECT * pRetRects = (RECT *) pRet->Buffer ;

   // initialize return list with viewlist rects
   DebugAssert((bool(ALLOC_SIZE > sizeof(RGNDATAHEADER) + viewList.rdh.nCount * sizeof(RECT)),"Too Big")) ;

   memcpy(pRet,&viewList, sizeof(RGNDATAHEADER) + viewList.rdh.nCount * sizeof(RECT)) ;

   for (DWORD i = 0 ; i < clipList.rdh.nCount ; i++)
   {
      for (DWORD j = 0 ; j < pRet->rdh.nCount && j < maxRects; j++)
      {
         RECT rectList[10] ; // 9 max, possible copy from 10th on 
                             //   boundary condition
         rectList[0] = pRetRects[j] ;

         DWORD nRects = SubtractRect(rectList,pClipRects[i]) ;
         // insert list in place of single element
         if (nRects == 0)
         {
            memcpy(&pRetRects[j],&pRetRects[j+1],
                   (pRet->rdh.nCount - j) * sizeof(RECT)) ;
         }
         else
         {
            DWORD x ;
            // make room for nRects elements. Note that we're overwriting
            // the original.
            RECT * src  = &pRetRects[pRet->rdh.nCount-1] ;
            RECT * dest = &pRetRects[(pRet->rdh.nCount-1)+(nRects-1)] ;
            for (x = 0 ; x < pRet->rdh.nCount - j ; x++)
               *dest-- = *src-- ;

            for (x = 0 ; x < nRects && (x+j) < maxRects; x++)
            {
               DebugAssert((bool((char *)&pRetRects[x+j] < (char *) pRet + ALLOC_SIZE),"Off the end\n" )) ;
               pRetRects[x+j] = rectList[x] ;
            }
         }

         // adjust nCount
         pRet->rdh.nCount += (DWORD) (nRects-1)  ;

         // adjust i (rectangles we just added are guaranteed to 
         // be excluded from the current clip rect).
         j += (DWORD)(nRects -1);
      }
   }
   DebugAssert((bool(p[ALLOC_SIZE -1] == 0xa5),"Sentinel got clobbered\n")) ;
   DebugAssert((bool(pRet->rdh.nCount < maxRects),"Too many rects (%d) max = %d\n",pRet->rdh.nCount, maxRects)) ; 

   return pRet ;  
}
// EOF
